# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: sarus_data_spec/protobuf/scalar.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='sarus_data_spec/protobuf/scalar.proto',
  package='sarus_data_spec',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n%sarus_data_spec/protobuf/scalar.proto\x12\x0fsarus_data_spec\"\xd0\x0f\n\x06Scalar\x12\x0c\n\x04uuid\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0b\n\x03\x64oc\x18\x03 \x01(\t\x12*\n\x04spec\x18\x04 \x01(\x0b\x32\x1c.sarus_data_spec.Scalar.Spec\x12;\n\nproperties\x18\x05 \x03(\x0b\x32\'.sarus_data_spec.Scalar.PropertiesEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xbd\x02\n\x04Spec\x12:\n\x0btransformed\x18\x01 \x01(\x0b\x32#.sarus_data_spec.Scalar.TransformedH\x00\x12.\n\x05model\x18\x02 \x01(\x0b\x32\x1d.sarus_data_spec.Scalar.ModelH\x00\x12\x43\n\x0eprivacy_params\x18\x03 \x01(\x0b\x32).sarus_data_spec.Scalar.PrivacyParametersH\x00\x12\x39\n\x0brandom_seed\x18\x04 \x01(\x0b\x32\".sarus_data_spec.Scalar.RandomSeedH\x00\x12\x41\n\x0fsynthetic_model\x18\x05 \x01(\x0b\x32&.sarus_data_spec.Scalar.SyntheticModelH\x00\x42\x06\n\x04spec\x1a\xbc\x01\n\x0bTransformed\x12\x11\n\ttransform\x18\x01 \x01(\t\x12\x11\n\targuments\x18\x02 \x03(\t\x12P\n\x0fnamed_arguments\x18\x03 \x03(\x0b\x32\x37.sarus_data_spec.Scalar.Transformed.NamedArgumentsEntry\x1a\x35\n\x13NamedArgumentsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xd3\x08\n\x05Model\x12\x11\n\targuments\x18\x01 \x01(\x0c\x12\x17\n\x0fnamed_arguments\x18\x02 \x01(\x0c\x12=\n\x0bmodel_class\x18\x03 \x01(\x0e\x32(.sarus_data_spec.Scalar.Model.ModelClass\"\xde\x07\n\nModelClass\x12\x0c\n\x08TF_KERAS\x10\x00\x12\n\n\x06SK_SVC\x10\x01\x12\r\n\tSK_ONEHOT\x10\x02\x12\n\n\x06SK_PCA\x10\x03\x12\x1b\n\x17SK_AFFINITY_PROPAGATION\x10\x04\x12\x1f\n\x1bSK_AGGLOMERATIVE_CLUSTERING\x10\x05\x12\x0c\n\x08SK_BIRCH\x10\x06\x12\r\n\tSK_DBSCAN\x10\x07\x12\x1c\n\x18SK_FEATURE_AGGLOMERATION\x10\x08\x12\r\n\tSK_KMEANS\x10\t\x12\x17\n\x13SK_MINIBATCH_KMEANS\x10\n\x12\x11\n\rSK_MEAN_SHIFT\x10\x0b\x12\r\n\tSK_OPTICS\x10\x0c\x12\x1a\n\x16SK_SPECTRAL_CLUSTERING\x10\r\x12\x1c\n\x18SK_SPECTRAL_BICLUSTERING\x10\x0e\x12\x1c\n\x18SK_SPECTRAL_COCLUSTERING\x10\x0f\x12\x1a\n\x16SK_ADABOOST_CLASSIFIER\x10<\x12\x19\n\x15SK_ADABOOST_REGRESSOR\x10=\x12\x19\n\x15SK_BAGGING_CLASSIFIER\x10>\x12\x18\n\x14SK_BAGGING_REGRESSOR\x10?\x12\x1c\n\x18SK_EXTRA_TREES_REGRESSOR\x10@\x12\x1d\n\x19SK_EXTRA_TREES_CLASSIFIER\x10\x41\x12#\n\x1fSK_GRADIENT_BOOSTING_CLASSIFIER\x10\x42\x12\"\n\x1eSK_GRADIENT_BOOSTING_REGRESSOR\x10\x43\x12\x17\n\x13SK_ISOLATION_FOREST\x10\x44\x12\x1f\n\x1bSK_RANDOM_FOREST_CLASSIFIER\x10\x45\x12\x1e\n\x1aSK_RANDOM_FOREST_REGRESSOR\x10\x46\x12\x1d\n\x19SK_RANDOM_TREES_EMBEDDING\x10G\x12\x1a\n\x16SK_STACKING_CLASSIFIER\x10H\x12\x19\n\x15SK_STACKING_REGRESSOR\x10I\x12\x18\n\x14SK_VOTING_CLASSIFIER\x10J\x12\x17\n\x13SK_VOTING_REGRESSOR\x10K\x12\'\n#SK_HIST_GRADIENT_BOOSTING_REGRESSOR\x10L\x12(\n$SK_HIST_GRADIENT_BOOSTING_CLASSIFIER\x10M\x12 \n\x1cSK_REPEATED_STRATIFIED_KFOLD\x10P\x12\x12\n\x0eXGB_CLASSIFIER\x10\\\x12\x14\n\x10SK_LABEL_ENCODER\x10\x62\x12\x0c\n\x08SK_KFOLD\x10\x63\x1a}\n\x11PrivacyParameters\x12?\n\x06points\x18\x01 \x03(\x0b\x32/.sarus_data_spec.Scalar.PrivacyParameters.Point\x1a\'\n\x05Point\x12\x0f\n\x07\x65psilon\x18\x01 \x01(\x01\x12\r\n\x05\x64\x65lta\x18\x02 \x01(\x01\x1a\x1b\n\nRandomSeed\x12\r\n\x05value\x18\x01 \x01(\x05\x1a\x10\n\x0eSyntheticModelb\x06proto3'
)



_SCALAR_MODEL_MODELCLASS = _descriptor.EnumDescriptor(
  name='ModelClass',
  full_name='sarus_data_spec.Scalar.Model.ModelClass',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='TF_KERAS', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_SVC', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_ONEHOT', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_PCA', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_AFFINITY_PROPAGATION', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_AGGLOMERATIVE_CLUSTERING', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_BIRCH', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_DBSCAN', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_FEATURE_AGGLOMERATION', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_KMEANS', index=9, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_MINIBATCH_KMEANS', index=10, number=10,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_MEAN_SHIFT', index=11, number=11,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_OPTICS', index=12, number=12,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_SPECTRAL_CLUSTERING', index=13, number=13,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_SPECTRAL_BICLUSTERING', index=14, number=14,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_SPECTRAL_COCLUSTERING', index=15, number=15,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_ADABOOST_CLASSIFIER', index=16, number=60,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_ADABOOST_REGRESSOR', index=17, number=61,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_BAGGING_CLASSIFIER', index=18, number=62,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_BAGGING_REGRESSOR', index=19, number=63,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_EXTRA_TREES_REGRESSOR', index=20, number=64,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_EXTRA_TREES_CLASSIFIER', index=21, number=65,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_GRADIENT_BOOSTING_CLASSIFIER', index=22, number=66,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_GRADIENT_BOOSTING_REGRESSOR', index=23, number=67,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_ISOLATION_FOREST', index=24, number=68,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_RANDOM_FOREST_CLASSIFIER', index=25, number=69,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_RANDOM_FOREST_REGRESSOR', index=26, number=70,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_RANDOM_TREES_EMBEDDING', index=27, number=71,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_STACKING_CLASSIFIER', index=28, number=72,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_STACKING_REGRESSOR', index=29, number=73,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_VOTING_CLASSIFIER', index=30, number=74,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_VOTING_REGRESSOR', index=31, number=75,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_HIST_GRADIENT_BOOSTING_REGRESSOR', index=32, number=76,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_HIST_GRADIENT_BOOSTING_CLASSIFIER', index=33, number=77,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_REPEATED_STRATIFIED_KFOLD', index=34, number=80,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='XGB_CLASSIFIER', index=35, number=92,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_LABEL_ENCODER', index=36, number=98,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SK_KFOLD', index=37, number=99,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=895,
  serialized_end=1885,
)
_sym_db.RegisterEnumDescriptor(_SCALAR_MODEL_MODELCLASS)


_SCALAR_PROPERTIESENTRY = _descriptor.Descriptor(
  name='PropertiesEntry',
  full_name='sarus_data_spec.Scalar.PropertiesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='sarus_data_spec.Scalar.PropertiesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='sarus_data_spec.Scalar.PropertiesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=215,
  serialized_end=264,
)

_SCALAR_SPEC = _descriptor.Descriptor(
  name='Spec',
  full_name='sarus_data_spec.Scalar.Spec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='transformed', full_name='sarus_data_spec.Scalar.Spec.transformed', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='model', full_name='sarus_data_spec.Scalar.Spec.model', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='privacy_params', full_name='sarus_data_spec.Scalar.Spec.privacy_params', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='random_seed', full_name='sarus_data_spec.Scalar.Spec.random_seed', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='synthetic_model', full_name='sarus_data_spec.Scalar.Spec.synthetic_model', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='spec', full_name='sarus_data_spec.Scalar.Spec.spec',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=267,
  serialized_end=584,
)

_SCALAR_TRANSFORMED_NAMEDARGUMENTSENTRY = _descriptor.Descriptor(
  name='NamedArgumentsEntry',
  full_name='sarus_data_spec.Scalar.Transformed.NamedArgumentsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='sarus_data_spec.Scalar.Transformed.NamedArgumentsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='sarus_data_spec.Scalar.Transformed.NamedArgumentsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=722,
  serialized_end=775,
)

_SCALAR_TRANSFORMED = _descriptor.Descriptor(
  name='Transformed',
  full_name='sarus_data_spec.Scalar.Transformed',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='transform', full_name='sarus_data_spec.Scalar.Transformed.transform', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='arguments', full_name='sarus_data_spec.Scalar.Transformed.arguments', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='named_arguments', full_name='sarus_data_spec.Scalar.Transformed.named_arguments', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_SCALAR_TRANSFORMED_NAMEDARGUMENTSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=587,
  serialized_end=775,
)

_SCALAR_MODEL = _descriptor.Descriptor(
  name='Model',
  full_name='sarus_data_spec.Scalar.Model',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='arguments', full_name='sarus_data_spec.Scalar.Model.arguments', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='named_arguments', full_name='sarus_data_spec.Scalar.Model.named_arguments', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='model_class', full_name='sarus_data_spec.Scalar.Model.model_class', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _SCALAR_MODEL_MODELCLASS,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=778,
  serialized_end=1885,
)

_SCALAR_PRIVACYPARAMETERS_POINT = _descriptor.Descriptor(
  name='Point',
  full_name='sarus_data_spec.Scalar.PrivacyParameters.Point',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='epsilon', full_name='sarus_data_spec.Scalar.PrivacyParameters.Point.epsilon', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='delta', full_name='sarus_data_spec.Scalar.PrivacyParameters.Point.delta', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1973,
  serialized_end=2012,
)

_SCALAR_PRIVACYPARAMETERS = _descriptor.Descriptor(
  name='PrivacyParameters',
  full_name='sarus_data_spec.Scalar.PrivacyParameters',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='points', full_name='sarus_data_spec.Scalar.PrivacyParameters.points', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_SCALAR_PRIVACYPARAMETERS_POINT, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1887,
  serialized_end=2012,
)

_SCALAR_RANDOMSEED = _descriptor.Descriptor(
  name='RandomSeed',
  full_name='sarus_data_spec.Scalar.RandomSeed',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='sarus_data_spec.Scalar.RandomSeed.value', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2014,
  serialized_end=2041,
)

_SCALAR_SYNTHETICMODEL = _descriptor.Descriptor(
  name='SyntheticModel',
  full_name='sarus_data_spec.Scalar.SyntheticModel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2043,
  serialized_end=2059,
)

_SCALAR = _descriptor.Descriptor(
  name='Scalar',
  full_name='sarus_data_spec.Scalar',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='uuid', full_name='sarus_data_spec.Scalar.uuid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='sarus_data_spec.Scalar.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='doc', full_name='sarus_data_spec.Scalar.doc', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='spec', full_name='sarus_data_spec.Scalar.spec', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='properties', full_name='sarus_data_spec.Scalar.properties', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_SCALAR_PROPERTIESENTRY, _SCALAR_SPEC, _SCALAR_TRANSFORMED, _SCALAR_MODEL, _SCALAR_PRIVACYPARAMETERS, _SCALAR_RANDOMSEED, _SCALAR_SYNTHETICMODEL, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=59,
  serialized_end=2059,
)

_SCALAR_PROPERTIESENTRY.containing_type = _SCALAR
_SCALAR_SPEC.fields_by_name['transformed'].message_type = _SCALAR_TRANSFORMED
_SCALAR_SPEC.fields_by_name['model'].message_type = _SCALAR_MODEL
_SCALAR_SPEC.fields_by_name['privacy_params'].message_type = _SCALAR_PRIVACYPARAMETERS
_SCALAR_SPEC.fields_by_name['random_seed'].message_type = _SCALAR_RANDOMSEED
_SCALAR_SPEC.fields_by_name['synthetic_model'].message_type = _SCALAR_SYNTHETICMODEL
_SCALAR_SPEC.containing_type = _SCALAR
_SCALAR_SPEC.oneofs_by_name['spec'].fields.append(
  _SCALAR_SPEC.fields_by_name['transformed'])
_SCALAR_SPEC.fields_by_name['transformed'].containing_oneof = _SCALAR_SPEC.oneofs_by_name['spec']
_SCALAR_SPEC.oneofs_by_name['spec'].fields.append(
  _SCALAR_SPEC.fields_by_name['model'])
_SCALAR_SPEC.fields_by_name['model'].containing_oneof = _SCALAR_SPEC.oneofs_by_name['spec']
_SCALAR_SPEC.oneofs_by_name['spec'].fields.append(
  _SCALAR_SPEC.fields_by_name['privacy_params'])
_SCALAR_SPEC.fields_by_name['privacy_params'].containing_oneof = _SCALAR_SPEC.oneofs_by_name['spec']
_SCALAR_SPEC.oneofs_by_name['spec'].fields.append(
  _SCALAR_SPEC.fields_by_name['random_seed'])
_SCALAR_SPEC.fields_by_name['random_seed'].containing_oneof = _SCALAR_SPEC.oneofs_by_name['spec']
_SCALAR_SPEC.oneofs_by_name['spec'].fields.append(
  _SCALAR_SPEC.fields_by_name['synthetic_model'])
_SCALAR_SPEC.fields_by_name['synthetic_model'].containing_oneof = _SCALAR_SPEC.oneofs_by_name['spec']
_SCALAR_TRANSFORMED_NAMEDARGUMENTSENTRY.containing_type = _SCALAR_TRANSFORMED
_SCALAR_TRANSFORMED.fields_by_name['named_arguments'].message_type = _SCALAR_TRANSFORMED_NAMEDARGUMENTSENTRY
_SCALAR_TRANSFORMED.containing_type = _SCALAR
_SCALAR_MODEL.fields_by_name['model_class'].enum_type = _SCALAR_MODEL_MODELCLASS
_SCALAR_MODEL.containing_type = _SCALAR
_SCALAR_MODEL_MODELCLASS.containing_type = _SCALAR_MODEL
_SCALAR_PRIVACYPARAMETERS_POINT.containing_type = _SCALAR_PRIVACYPARAMETERS
_SCALAR_PRIVACYPARAMETERS.fields_by_name['points'].message_type = _SCALAR_PRIVACYPARAMETERS_POINT
_SCALAR_PRIVACYPARAMETERS.containing_type = _SCALAR
_SCALAR_RANDOMSEED.containing_type = _SCALAR
_SCALAR_SYNTHETICMODEL.containing_type = _SCALAR
_SCALAR.fields_by_name['spec'].message_type = _SCALAR_SPEC
_SCALAR.fields_by_name['properties'].message_type = _SCALAR_PROPERTIESENTRY
DESCRIPTOR.message_types_by_name['Scalar'] = _SCALAR
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Scalar = _reflection.GeneratedProtocolMessageType('Scalar', (_message.Message,), {

  'PropertiesEntry' : _reflection.GeneratedProtocolMessageType('PropertiesEntry', (_message.Message,), {
    'DESCRIPTOR' : _SCALAR_PROPERTIESENTRY,
    '__module__' : 'sarus_data_spec.protobuf.scalar_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Scalar.PropertiesEntry)
    })
  ,

  'Spec' : _reflection.GeneratedProtocolMessageType('Spec', (_message.Message,), {
    'DESCRIPTOR' : _SCALAR_SPEC,
    '__module__' : 'sarus_data_spec.protobuf.scalar_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Scalar.Spec)
    })
  ,

  'Transformed' : _reflection.GeneratedProtocolMessageType('Transformed', (_message.Message,), {

    'NamedArgumentsEntry' : _reflection.GeneratedProtocolMessageType('NamedArgumentsEntry', (_message.Message,), {
      'DESCRIPTOR' : _SCALAR_TRANSFORMED_NAMEDARGUMENTSENTRY,
      '__module__' : 'sarus_data_spec.protobuf.scalar_pb2'
      # @@protoc_insertion_point(class_scope:sarus_data_spec.Scalar.Transformed.NamedArgumentsEntry)
      })
    ,
    'DESCRIPTOR' : _SCALAR_TRANSFORMED,
    '__module__' : 'sarus_data_spec.protobuf.scalar_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Scalar.Transformed)
    })
  ,

  'Model' : _reflection.GeneratedProtocolMessageType('Model', (_message.Message,), {
    'DESCRIPTOR' : _SCALAR_MODEL,
    '__module__' : 'sarus_data_spec.protobuf.scalar_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Scalar.Model)
    })
  ,

  'PrivacyParameters' : _reflection.GeneratedProtocolMessageType('PrivacyParameters', (_message.Message,), {

    'Point' : _reflection.GeneratedProtocolMessageType('Point', (_message.Message,), {
      'DESCRIPTOR' : _SCALAR_PRIVACYPARAMETERS_POINT,
      '__module__' : 'sarus_data_spec.protobuf.scalar_pb2'
      # @@protoc_insertion_point(class_scope:sarus_data_spec.Scalar.PrivacyParameters.Point)
      })
    ,
    'DESCRIPTOR' : _SCALAR_PRIVACYPARAMETERS,
    '__module__' : 'sarus_data_spec.protobuf.scalar_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Scalar.PrivacyParameters)
    })
  ,

  'RandomSeed' : _reflection.GeneratedProtocolMessageType('RandomSeed', (_message.Message,), {
    'DESCRIPTOR' : _SCALAR_RANDOMSEED,
    '__module__' : 'sarus_data_spec.protobuf.scalar_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Scalar.RandomSeed)
    })
  ,

  'SyntheticModel' : _reflection.GeneratedProtocolMessageType('SyntheticModel', (_message.Message,), {
    'DESCRIPTOR' : _SCALAR_SYNTHETICMODEL,
    '__module__' : 'sarus_data_spec.protobuf.scalar_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Scalar.SyntheticModel)
    })
  ,
  'DESCRIPTOR' : _SCALAR,
  '__module__' : 'sarus_data_spec.protobuf.scalar_pb2'
  # @@protoc_insertion_point(class_scope:sarus_data_spec.Scalar)
  })
_sym_db.RegisterMessage(Scalar)
_sym_db.RegisterMessage(Scalar.PropertiesEntry)
_sym_db.RegisterMessage(Scalar.Spec)
_sym_db.RegisterMessage(Scalar.Transformed)
_sym_db.RegisterMessage(Scalar.Transformed.NamedArgumentsEntry)
_sym_db.RegisterMessage(Scalar.Model)
_sym_db.RegisterMessage(Scalar.PrivacyParameters)
_sym_db.RegisterMessage(Scalar.PrivacyParameters.Point)
_sym_db.RegisterMessage(Scalar.RandomSeed)
_sym_db.RegisterMessage(Scalar.SyntheticModel)


_SCALAR_PROPERTIESENTRY._options = None
_SCALAR_TRANSFORMED_NAMEDARGUMENTSENTRY._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: sarus_data_spec/protobuf/manager.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='sarus_data_spec/protobuf/manager.proto',
  package='sarus_data_spec',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n&sarus_data_spec/protobuf/manager.proto\x12\x0fsarus_data_spec\"\x88\x01\n\x07Manager\x12\x0c\n\x04uuid\x18\x01 \x01(\t\x12<\n\nproperties\x18\x02 \x03(\x0b\x32(.sarus_data_spec.Manager.PropertiesEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x62\x06proto3'
)




_MANAGER_PROPERTIESENTRY = _descriptor.Descriptor(
  name='PropertiesEntry',
  full_name='sarus_data_spec.Manager.PropertiesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='sarus_data_spec.Manager.PropertiesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='sarus_data_spec.Manager.PropertiesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=147,
  serialized_end=196,
)

_MANAGER = _descriptor.Descriptor(
  name='Manager',
  full_name='sarus_data_spec.Manager',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='uuid', full_name='sarus_data_spec.Manager.uuid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='properties', full_name='sarus_data_spec.Manager.properties', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_MANAGER_PROPERTIESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=60,
  serialized_end=196,
)

_MANAGER_PROPERTIESENTRY.containing_type = _MANAGER
_MANAGER.fields_by_name['properties'].message_type = _MANAGER_PROPERTIESENTRY
DESCRIPTOR.message_types_by_name['Manager'] = _MANAGER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Manager = _reflection.GeneratedProtocolMessageType('Manager', (_message.Message,), {

  'PropertiesEntry' : _reflection.GeneratedProtocolMessageType('PropertiesEntry', (_message.Message,), {
    'DESCRIPTOR' : _MANAGER_PROPERTIESENTRY,
    '__module__' : 'sarus_data_spec.protobuf.manager_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Manager.PropertiesEntry)
    })
  ,
  'DESCRIPTOR' : _MANAGER,
  '__module__' : 'sarus_data_spec.protobuf.manager_pb2'
  # @@protoc_insertion_point(class_scope:sarus_data_spec.Manager)
  })
_sym_db.RegisterMessage(Manager)
_sym_db.RegisterMessage(Manager.PropertiesEntry)


_MANAGER_PROPERTIESENTRY._options = None
# @@protoc_insertion_point(module_scope)



import pyarrow as pa

import sarus_data_spec.typing as st


def convert_record_batch(
    record_batch: pa.RecordBatch, _type: st.Type
) -> pa.Array:
    if str(_type.protobuf().WhichOneof('type')) not in ['struct', 'union']:
        return record_batch.column(0)

    names = list(_type.children().keys())
    if _type.protobuf().HasField('union'):
        # A Sarus Union is modelled a a Struct with an additional
        # `field_selected` field
        names.append('field_selected')

    return pa.StructArray.from_arrays(record_batch.columns, names=names)



import logging
import pickle as pkl
import traceback
import typing as t

from sarus_data_spec import typing as st
from sarus_data_spec.constants import SCALAR_TASK, ScalarCaching
from sarus_data_spec.manager.asyncio.worker.cache_scalar_computation import (
    CacheScalarComputation,
)
from sarus_data_spec.manager.asyncio.worker.worker_computation import (
    TypedWorkerManager,
    WorkerComputation,
)
from sarus_data_spec.scalar import Scalar
import sarus_data_spec.protobuf as sp
import sarus_data_spec.status as stt

logger = logging.getLogger(__name__)


class ValueComputation(WorkerComputation[t.Any]):
    """Class responsible for handling the computation
    of scalars."""

    task_name = SCALAR_TASK

    def __init__(
        self,
        manager: TypedWorkerManager,
        cache_scalar_computation: CacheScalarComputation,
    ) -> None:
        super().__init__(manager)
        self.cache_scalar_computation = cache_scalar_computation

    async def read_ready_result(
        self,
        dataspec: st.DataSpec,
        properties: t.Mapping[str, str],
        **kwargs: t.Any,
    ) -> t.Any:
        if self.manager().is_cached(dataspec):
            (
                cache_type,
                cache,
            ) = await self.cache_scalar_computation.task_result(dataspec)
            try:
                if cache_type == ScalarCaching.PICKLE.value:
                    with open(cache, "rb") as f:
                        data = pkl.load(f)

                else:
                    data = sp.python_proto_factory(cache, cache_type)
            except Exception as e:
                stt.error(
                    dataspec=dataspec,
                    manager=self.manager(),
                    task=self.task_name,
                    properties={
                        "message": traceback.format_exc(),
                        'relaunch': str(True),
                    },
                )
                stt.error(
                    dataspec=dataspec,
                    manager=self.manager(),
                    task=self.cache_scalar_computation.task_name,
                    properties={
                        "message": traceback.format_exc(),
                        'relaunch': str(True),
                    },
                )

                raise stt.DataSpecErrorStatus(
                    (True, traceback.format_exc())
                ) from e
            else:
                return data

        return await self.manager().async_value_op(
            scalar=t.cast(Scalar, dataspec)
        )

    async def prepare(self, dataspec: st.DataSpec) -> None:
        try:
            logger.debug(f'STARTED SCALAR {dataspec.uuid()}')

            if self.manager().is_cached(dataspec):
                await self.cache_scalar_computation.task_result(dataspec)
            else:
                await self.manager().async_prepare_parents(dataspec)
        except stt.DataSpecErrorStatus as exception:
            stt.error(
                dataspec=dataspec,
                manager=self.manager(),
                task=self.task_name,
                properties={
                    "message": traceback.format_exc(),
                    'relaunch': str(exception.relaunch),
                },
            )
            raise
        except Exception:
            stt.error(
                dataspec=dataspec,
                manager=self.manager(),
                task=self.task_name,
                properties={
                    "message": traceback.format_exc(),
                    'relaunch': str(False),
                },
            )
            raise stt.DataSpecErrorStatus((False, traceback.format_exc()))
        else:
            logging.debug(f'FINISHED SCALAR {dataspec.uuid()}')
            stt.ready(
                dataspec=dataspec,
                manager=self.manager(),
                task=self.task_name,
            )



import logging
import typing as t

import pyarrow as pa

from sarus_data_spec.manager.asyncio.base import BaseComputation, T
from sarus_data_spec.manager.typing import DelegatedComputation, Manager
import sarus_data_spec.status as stt
import sarus_data_spec.typing as st

logger = logging.getLogger(__name__)


class TypedApiManager(Manager, t.Protocol):
    async def async_to_arrow_op(
        self, dataset: st.Dataset, batch_size: int
    ) -> t.AsyncIterator[pa.RecordBatch]:
        ...

    async def async_value_op(self, scalar: st.Scalar) -> st.DataSpecValue:
        ...

    def computation_timeout(self, dataspec: st.DataSpec) -> int:
        ...

    def computation_max_delay(self, dataspec: st.DataSpec) -> int:
        ...


class ApiComputation(BaseComputation[T], DelegatedComputation):
    def __init__(self, manager: TypedApiManager):
        self._manager: TypedApiManager = manager

    def manager(self) -> TypedApiManager:
        return self._manager

    async def read_ready_result(
        self,
        dataspec: st.DataSpec,
        properties: t.Mapping[str, str],
        **kwargs: t.Any,
    ) -> T:
        raise NotImplementedError

    async def pending(self, dataspec: st.DataSpec) -> st.Status:
        """if the status of a task is pending, delegation has been
        already done, so the manager just waits for the task to
        be completed"""

        stage = await self.wait_for_computation(
            dataspec=dataspec,
            current_stage='pending',
            timeout=self.manager().computation_timeout(dataspec),
            max_delay=self.manager().computation_max_delay(dataspec),
        )

        if stage.pending():
            # TODO push error status to worker ?
            stt.error(
                dataspec=dataspec,
                manager=dataspec.manager(),
                task=self.task_name,
                properties={
                    "message": (
                        "TimeOutError: Pending time out for task"
                        f" {self.task_name} on dataspec {dataspec}"
                    ),
                    'relaunch': str(True),
                },
            )
            raise stt.DataSpecErrorStatus(
                (
                    True,
                    "Pending time out for task"
                    f" {self.task_name} on dataspec {dataspec}",
                )
            )

        return await self.complete_task(dataspec)

    async def processing(self, dataspec: st.DataSpec) -> st.Status:
        """If processing, wait for the task to be ready.
        Such a case can happen if another manager has taken the computation
        of the task. After a given timeout, an error is raised.
        """

        stage = await self.wait_for_computation(
            dataspec=dataspec,
            current_stage='processing',
            timeout=self.manager().computation_timeout(dataspec),
            max_delay=self.manager().computation_max_delay(dataspec),
        )
        if stage.processing() or stage.pending():
            # TODO push error status to worker ?
            stt.error(
                dataspec=dataspec,
                manager=dataspec.manager(),
                task=self.task_name,
                properties={
                    "message": (
                        "TimeOutError: Processing time out for task"
                        f" {self.task_name} on dataspec {dataspec}"
                    ),
                    'relaunch': str(True),
                },
            )
            raise stt.DataSpecErrorStatus(
                (
                    True,
                    "Processing time out for task"
                    f" {self.task_name} on dataspec {dataspec}",
                )
            )

        # if the stage is an error, it is complete_task
        # that decides what to do
        return await self.complete_task(dataspec)



from typing import Any

import numpy as np


async def np_array(*args: Any, **kwargs: Any) -> Any:
    return np.array(*args, **kwargs)


async def np_astype(val: Any, *args: Any, **kwargs: Any) -> Any:
    return val.astype(*args, **kwargs)


async def np_ceil(*args: Any, **kwargs: Any) -> Any:
    return np.ceil(*args, **kwargs)


async def np_floor(*args: Any, **kwargs: Any) -> Any:
    return np.floor(*args, **kwargs)


async def np_mean(*args: Any, **kwargs: Any) -> Any:
    return np.mean(*args, **kwargs)


async def np_rand(*args: int) -> Any:
    return np.random.rand(*args)


async def np_std(*args: Any, **kwargs: Any) -> Any:
    return np.std(*args, **kwargs)



from __future__ import annotations

from typing import Any

try:
    from imblearn import over_sampling, pipeline, under_sampling
except ModuleNotFoundError:
    pass  # error message in typing.py


async def imb_pipeline(
    *args: Any,
    **kwargs: Any,
) -> Any:
    return pipeline.Pipeline(*args, **kwargs)


async def imb_random_under_sampler(
    *args: Any,
    **kwargs: Any,
) -> Any:
    return under_sampling.RandomUnderSampler(*args, **kwargs)


async def imb_smotenc(
    *args: Any,
    **kwargs: Any,
) -> Any:
    return over_sampling.SMOTENC(*args, **kwargs)
